/**
 * @typedef {object} WordList
 * @property {{[key:string]:string}} [macros]
 * @property {(Topic|Word)[]} words
 */
/**
 * @typedef {object} Topic
 * @property {"topic"} type
 * @property {string} name
 * @property {Word[]} words
 */
/**
 * @typedef {object} Word
 * @property {"word"} [type]
 * @property {string} word
 * @property {string} [context]
 * @property {string} [description]
 * @property {string[]} [synonyms]
 */
/** @type {Promise<WordList>} */
let json = (async()=>{
	let response = await fetch("./Skribbl.json");
	return await response.json();
})();
let words = (async()=>{
	let words = (await json).words.map(wordOrTopic=>wordOrTopic.type=="topic"?wordOrTopic.words:[wordOrTopic]).flat();
	return words;
})();
/**
 * Wrapper for the word list specified in `Skribbl.json`.
 */
export default class SkribblWords {
	/**
	 * Returns a random word from the list.
	 */
	static async get(){
		let array = await words;
		return array[Math.floor(Math.random()*array.length)];
	}

	/**
	 * @type {Promise<{[key:string]:string}>}
	 * @readonly
	 */
	static get macros(){
		return (async ()=>{
			return (await json).macros;
		})();
	}

	/**
	 * Checks whether a given guess is close to the given word. A guess counts as close, if one letter is wrong,
	 * two letters are swapped or the guess has one letter too much or too little.
	 * @param {string} guess
	 * @param {string} word
	 */
	 static _isClose(guess,word) {
		guess = guess.toLowerCase().replace(/-/g," ").replace(/'/g,'').normalize("NFD").replace(/[\u0300-\u0307\u0309-\u036f]/g, "");
		word = word.toLowerCase().replace(/-/g," ").replace(/'/g,'').normalize("NFD").replace(/[\u0300-\u0307\u0309-\u036f]/g, "");
		const wordArray = Array.from(word);
		const guessArray = Array.from(guess);
	
		//if equal
		if (guess == word) { 
			return true;
		}
		
		//either one letter wrong or two letters swapped
		if (guess.length == word.length) {
			//Counts the mistakes and their position
			let errorCounter = 0;
			let errorPos = 0;
			for (var i = 0; i < wordArray.length; i++) {
				if (wordArray[i] != guessArray[i]) {
					if (errorCounter == 0) {
						errorPos = i;
					} if (errorCounter == 1) { //if a second mistake occurs, either the letters are swapped or it is not correct
												//but there could be a third mistake, so in the case of a swap, true is not directly returned.
						if ((wordArray[i] != guessArray[errorPos]) || (wordArray[errorPos] != guessArray[i])) {
							return false;
						}
					} if (errorCounter >= 2) { //with two or more mistakes, the word is not close
						return false;
					}
					errorCounter++;
				}
			}
			//if it hasnt returned false by now, the guess is close
			return true;
		}
	
		//if one letter too much
		if (guess.length - 1 == word.length)  {
			let errorCounter = 0; //also the offset
			for (var i = 0; i < wordArray.length; i++) {
				if (wordArray[i] != guessArray[i + errorCounter]) {
					errorCounter++;
					if (errorCounter >= 2) {
						return false;
					}
				}
			}
			return true;
		}
	
		//if one letter too little
		if (guess.length + 1 == word.length) {
			let errorCounter = 0; //also the offset
			for (var i = 0; i < guessArray.length; i++) {
				if (wordArray[i + errorCounter] != guessArray[i]) {
					errorCounter++;
					if (errorCounter >= 2) {
						return false;
					}
				}
			}
			return true;
		}
	
		return false;
	}

	/**
	 * Checks if the guess is correct. It is not case-sensitive.
	 * @param {string} guess
	 * @param {string} word
	 */
	static _isCorrect(guess, word) {
		guess = guess.toLowerCase().replace(/-/g," ").replace(/'/g,'').normalize("NFD").replace(/[\u0300-\u0307\u0309-\u036f]/g, "");
		word = word.toLowerCase().replace(/-/g," ").replace(/'/g,'').normalize("NFD").replace(/[\u0300-\u0307\u0309-\u036f]/g, "");
		return guess==word;
	}
}