import SkribblClient from "../logic/SkribblClient.js";
import {CustomElement} from "../util/Util.js";

/**
 * Custom element `<skribbl-chat>` for the chat used to guess words and see others guesses.
 * @todo better scroll functionality (stay stickied to bottom unless specifically scrolled up, subtle css effects where stuff is hidden from view)
 * @todo allow cycling through previous guesses using the arrow keys, like in a terminal
 * @todo write to the chat which player is drawing now
 */
export default class SkribblChat extends CustomElement {
	/**
	 * @param {SkribblClient} client
	 */
	constructor(client){
		super();
		this.attachShadow({mode:"open"});
		this.shadowRoot.innerHTML = `
			<style>
				:host {
					display: flex;
					flex-direction: column;
					background: #ffffff;
					border-radius: 5px;
					padding: 5px;
				}
				#messages-outer-container {
					height: 100%;
					flex-shrink: 1;
					position: relative;
				}
				#messages-inner-container {
					position: absolute;
					width: 100%;
					height: 100%;
					overflow-y: auto;
				}
				#input-container {
					flex-shrink: 0;
					padding: 3px 5px 0 5px;
					border-top: 1px solid #cfcfcf;
				}
				#input {
					width: 100%;
					box-sizing: border-box;
					border: 1px solid transparent;
					border-bottom-color: #80808080;
					color: #808080;
					transition: border-bottom-color 0.2s linear, color 0.2s linear;
					font-size: 1.15em;
				}
				#input:focus {
					outline: none;
					border-bottom-color: #000000;
					color: #000000;
				}
			</style>
			<div id="messages-outer-container">
				<div id="messages-inner-container"></div>
			</div>
			<div id="input-container">
				<input id="input" type="text" placeholder="your guess here!">
			</div>
		`;
		this._client = client;
		this._messageDiv = this.shadowRoot.getElementById("messages-inner-container");
		/** @type {HTMLInputElement} */// @ts-ignore
		this._input = this.shadowRoot.getElementById("input");
		this._input.addEventListener("keydown",e=>{
			if (e.key=="Enter"&&this._input.value!==""){
				this._client.sendGuess(this._input.value);
				this._input.value = "";
			}
		});
		client.onGuess(guessData=>{
			let playerName = this._client.players.value[guessData.player].name;
			this._messageDiv.appendChild(new SkribblGuess(playerName,guessData));
		},{onlyWhen:this.connected});
		client.onWordReveal(message=>{
			// TODO reveal the word in the chat
		},{onlyWhen:this.connected});
	}
}
/**
 * Custom element `<skribbl-guess>` that appears in the chat whenever someone tries to guess the word.
 * @todo display message when the word was close
 * @todo refer to the player with "you" when he guesses the word, not using his name
 */
export class SkribblGuess extends HTMLElement {
	/**
	 * @param {string} playerName
	 * @param {import("../logic/SkribblClient.js").GuessResponse} guess
	 */
	constructor(playerName,guess){
		super();
		this.attachShadow({mode:"open"});
		this.shadowRoot.innerHTML = `
			<style>
				:host {
					display: block;
				}
			</style>
			<b id="b1"></b><span id="span1"></span>
			<br><b id="b2" style="color:#649E64"></b><span id="span2" style="color:#649E64"></span>
		`;
		this._playerName = playerName;
		this._guess = guess;
		this.shadowRoot.querySelector("#b1").textContent = playerName;
		// seems unnecessary, I know. but for some reason IntelliSense won't correctly infer the type in the else block without the `==true`
		if (guess.correct==true){
			this.shadowRoot.querySelector("#span1").textContent = " has guessed the word!";
			this.style.color = "#00bf00";
		}else{
			this.shadowRoot.querySelector("#span1").textContent = `: ${guess.word}`;
		}

		if ((guess.correct!=true) && (guess.close==true)) {
			this.shadowRoot.querySelector("#b2").textContent = "'" + guess.word + "'";
			this.shadowRoot.querySelector("#span2").textContent = " is close!";
		}
	}

	/** @readonly */
	get playerName(){
		return this._playerName;
	}

	/** @readonly */
	get word(){
		return this._guess.word;
	}
}
customElements.define("skribbl-chat",SkribblChat);
customElements.define("skribbl-guess",SkribblGuess);